/* AppController.m
 * Taskmaster
 * Copyright (C) 2003 Arthur Lewis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "AppController.h"
#define TODOINDEX 3

@implementation AppController

-(id) init {
    
    // set global variables
    
    dirPath = [[NSString stringWithCString: "~/Documents/TaskMaster/"] stringByExpandingTildeInPath];
    [dirPath retain];
    path = [dirPath stringByAppendingPathComponent:@"TaskDB"];
    [path retain];

    today = [NSCalendarDate calendarDate];
    currentDate = today;
    [today retain];
    
    manager = [NSFileManager defaultManager];
    [manager retain];

    
    BOOL isDir;
    BOOL dirExists;
    BOOL shouldCreateData;

    // Checking for existing data
    if ([manager fileExistsAtPath:dirPath isDirectory:&isDir]) {

        if (!isDir) {
            NSLog(@"A file exists where your directory should be. What gives?");
            return nil;
        }
        
        // Load from file
        else {
            if ([manager fileExistsAtPath:path]) {
                NSLog (@"Loading data from file");
                if (self = [NSKeyedUnarchiver unarchiveObjectWithFile:path]) {
                    path = [[NSString stringWithCString: "~/Documents/TaskMaster/TaskDB"]
                        stringByExpandingTildeInPath];
                    [path retain];
                    [self retain];
                    today = [NSCalendarDate calendarDate];
                    currentDate = today;
                    
                    // Load today into the currentDay window
                    if ([days count] > 0) currentDay = [days objectForKey:[self objectNameFromDate]];
                    if (currentDay == nil) {
                        currentDay = [[Day alloc] init];
                        [days setObject:currentDay forKey:[self objectNameFromDate]];
                    }
                    currentDate = [currentDay getDate];
                }
                else return nil;
            }

            //Create data file and new data
            else {
                self = [self initFromScratch];
                if ([manager createFileAtPath:path contents:nil attributes:nil]) {
                    NSLog (path);
                }
                else {
                    NSLog(@"Could not create file");
                    return nil;
                }
                
            }

        }
    }
    // Create directory, data file, and new data
    else {
        self = [self initFromScratch];
        if (([manager createDirectoryAtPath:dirPath attributes:nil]) ||
            ([manager createFileAtPath:path contents:nil attributes:nil])) {
            NSLog(dirPath);
        }
        else {
            NSLog(@"Could not create file or directory");
            return nil;
        }
    }


    if (self == nil) return self;

    [days retain];
    [DB retain];
    [currentDay retain];
    [currentDate retain];
    openingDate = currentDate;
    [openingDate retain];

    tasksForToday = [currentDay getTasks];
    allTasks = [DB getTasks];

    [tasksForToday retain];
    [allTasks retain];
    today = [currentDay getDate];

    if ([self save]) {
        return self;
    }
    else return nil;
}

- (id)initFromScratch {
    
    if (self = [super init]) {
        DB = [[TaskDB alloc] init];
        days = [[NSMutableDictionary alloc] init];
        currentDay = [[Day alloc] init];
        [days setObject:currentDay forKey:[self objectNameFromDate]];
    }
    return self;
}
- (void)awakeFromNib {
    
    [textField setObjectValue:currentDate];

}



- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender {
    return YES;
}

// Methods for saving and opening data
- (BOOL)save {
    if ([NSKeyedArchiver archiveRootObject:self toFile:path]) {
        NSLog(@"Saved successfuly");
        NSLog(path);
        return YES;
    }
    else NSLog(@"Wanh, wanh");
    return NO;
}

-(void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:DB forKey:@"DB"];
    [coder encodeObject:days forKey:@"Days"];
}

-(id)initWithCoder:(NSCoder *)coder {
    DB = [coder decodeObjectForKey:@"DB"];
    days = [coder decodeObjectForKey:@"Days"];
    [DB retain];
    [days retain];
    return self;
}

- (NSString *)objectNameFromDate {
    NSString *filename = [currentDate descriptionWithCalendarFormat:@"%m.%d.%Y"];
    return filename;
}

-(IBAction)addButtonPressed:(NSEvent *)event {
    Task * newTask = [[Task alloc] init];
    [DB addTask:newTask];
    [dbTable reloadData];
    [dbTable selectRow:0 byExtendingSelection:NO];
    NSLog(@"New Task!!");
    [self save];
}

- (IBAction)addSelectedTask:(NSEvent *)event {
    NSLog(@"You pushed da button!");
    if ([dbTable selectedRow] < 0) return;
    
    Task *newTask = [allTasks objectAtIndex:[dbTable selectedRow]];
    if ([currentDay containsTask:newTask]) return; // no duplicates
    [newTask retain];
    [newTask setDateLastPerformed:currentDate];
    NSLog([newTask description]);
    [currentDay addTask:newTask];
    [dayTable reloadData];
    [dbTable reloadData];
    NSLog([currentDay description]);
    [self save];
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView {
    if (tableView == dbTable) {
        tasks = allTasks;
    }
    else tasks = tasksForToday;
    return [tasks count];
}

-(id)tableView:(NSTableView *)tableView
objectValueForTableColumn:(NSTableColumn *)tableColumn
           row:(int)row
{
    id value;
    NSMutableAttributedString *formattedValue;
    NSRange wholeString;
    NSColor *red = [NSColor redColor];
    
    if (tableView == dbTable) {
        tasks = allTasks;
        NSLog(@"allTasks");
    }
    else {
        tasks = tasksForToday;
        NSLog(@"tasksForToday");
    }
    NSString * identifier = [tableColumn identifier];
    Task * currentTask = [tasks objectAtIndex:row];
    value = [currentTask valueForKey:identifier];
    if ([identifier isEqualToString:@"name"]) {
        if ([currentTask getToDo]) {
            formattedValue = [[NSMutableAttributedString alloc] initWithString:value];
            wholeString.location = 0;
            wholeString.length = [formattedValue length];
            [formattedValue addAttribute:@"NSForegroundColorAttributeName" value:red range:wholeString];
            [red retain];
            
            [value release];
            value = formattedValue;
            [value retain];
        }
    }
    
    return value;
            
}

- (void)tableView:(NSTableView *)tableView
   setObjectValue:(id)object
   forTableColumn:(NSTableColumn *)tableColumn
              row:(int)row
{
    if (tableView == dbTable) {
        tasks = allTasks;
    }
    else tasks = tasksForToday;
    
    NSString * identifier = [tableColumn identifier];
    Task * currentTask = [tasks objectAtIndex:row];
    [currentTask takeValue:object forKey:identifier];
    [dbTable reloadData];
    [dayTable reloadData];
    [self save];
}


- (IBAction)removeSelectedTask:(NSEvent *)event {
    NSLog(@"You pushed da other button!");
    int row = [dayTable selectedRow];
    if (row < 0) return;
    [currentDay removeTask:row];
    printf("Removing row %i from dayTable\n", row);
    if (row == 0) row = 1;
    [dayTable selectRow:(row-1) byExtendingSelection:NO];
    [dayTable reloadData];
    [dbTable reloadData];
    [self save];
}

-(IBAction)dayUp:(id)sender {
    [self dayChange:[sender intValue]];
}

-(void)dayChange:(int)numDays {
    NSCalendarDate * newDate;
    newDate = [openingDate dateByAddingYears:0 months:0 days:numDays hours:0 minutes:0 seconds:0];
    [currentDate release];
    currentDate = newDate;
    [currentDate retain];
    currentDay = [days objectForKey:[self objectNameFromDate]];
    if (currentDay == nil) {
        currentDay = [[Day alloc] init];
        [days setObject:currentDay forKey:[self objectNameFromDate]];
    }
    [currentDay retain];
    [tasksForToday release];
    tasksForToday = [currentDay getTasks];
    [tasksForToday retain];
    [dayTable reloadData];
    [textField setObjectValue:currentDate];

}

@end
