/* Day.m
 * Taskmaster
 * Copyright (C) 2003 Arthur Lewis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "Day.h"
#import "Task.h"


@implementation Day
- (id)init {
    if (self = [super init]) {
        tasks = [[NSMutableArray alloc] init];
        [tasks retain];
    }
    return self;
}
-(id)initWithTasks:(NSMutableArray *)newTasks date:(NSCalendarDate *)newDate {
    if (self = [super init]) {
        [tasks release];
        [newTasks retain];
        tasks = newTasks;
//Does the Day object need to know its own date?  probably.

        if (newDate == nil) {
            thisDate = [NSCalendarDate calendarDate];
            [thisDate retain];
        }
        else {
            [thisDate release];
            [newDate retain];
            thisDate = newDate;
        }
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)coder {
    return [self initWithTasks:[coder decodeObjectForKey:@"Tasks"] date:[coder decodeObjectForKey:@"ThisDate"]];
}

-(void)encodeWithCoder:(NSCoder *)coder {
    [coder encodeObject:tasks forKey:@"Tasks"];
    [coder encodeObject:thisDate forKey:@"ThisDate"];
}

-(void)addTask:(Task *)newTask {
    [tasks insertObject:newTask atIndex:0];
    [newTask release]; // why does the book want this?
}

-(void)removeTask:(int)row {
    Task *sadTask = [tasks objectAtIndex:row];
    [sadTask undoDateLastPerformed];
    [tasks removeObjectAtIndex:row];
}

- (NSMutableArray *)getTasks {
    return tasks;
}

- (bool)containsTask:(Task *)newTask {
    return [tasks containsObject:newTask];
}
- (NSCalendarDate *)getDate {
    return thisDate;
}
- (void)setDate:(NSCalendarDate *)newDate {
    [thisDate release];
    [newDate retain];
    thisDate = newDate;
}
@end
